@@ -1,7 +1,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2007 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2010 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: database.c,v 1.81 2007/10/14 14:58:07 thib Exp thib $ */
+ /* $Id: database.c,v 1.83 2008-05-11 15:05:51 thib Exp $ */
 
 #include "fcron.h"
 
@@ -43,7 +43,7 @@ void run_queue_job(cl_t *line);
 void resize_exe_array(void);
 
 
-#ifndef HAVE_SETENV
+#if ! defined(HAVE_SETENV) || ! defined(HAVE_UNSETENV)
 char env_tz[PATH_LEN];
 #endif
 
@@ -109,10 +109,11 @@ switch_back_timezone(const char *orig_tz)
  * otherwise, sets TZ to orig_tz */
 {
     if ( orig_tz == NULL) {
-#ifdef HAVE_SETENV
+#ifdef HAVE_UNSETENV
 	unsetenv("TZ");
 #else
-	env_tz[0] = '\0';
+	snprintf(env_tz, sizeof(env_tz) - 1, "TZ=");
+	env_tz[sizeof(env_tz)-1] = '\0';
 	if ( putenv(env_tz) < 0 )
 	    error_e("could not flush env var TZ");	
 #endif
@@ -282,8 +283,15 @@ run_queue_job(cl_t *line)
 	resize_exe_array();
 
     exe_array[exe_num].e_line = line;
+    exe_num++;
 
-    run_job(&exe_array[exe_num++]);
+    /* run the job */
+    if ( run_job(&exe_array[exe_num-1]) != OK ) {
+	/* The job could not be run: remove it from the exe_array */
+	exe_array[exe_num-1].e_line->cl_numexe--;
+	exe_array[exe_num-1].e_line = NULL;
+	exe_num--;
+    }
 
 }
 
@@ -691,6 +699,7 @@ goto_beginning_next_period_periodical(cl_t *line, struct tm *ftime)
      * period of execution= a continuous interval of time during which
      *     the line is to be executed once and only once. */
 {
+    int max = 0;
 
     /* sanity check */
     if ( ! is_freq_periodically(line->cl_option) )
@@ -698,7 +707,7 @@ goto_beginning_next_period_periodical(cl_t *line, struct tm *ftime)
 
     
     /* number of days in ftime's month */
-    int max = get_nb_mdays(ftime->tm_year, ftime->tm_mon);
+    max = get_nb_mdays(ftime->tm_year, ftime->tm_mon);
 
     /* STEP 1: find the beginning of the next period without ensuring
      * there is no overflow (min>=60, hour>=24, etc) */
