@@ -1,7 +1,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2007 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2010 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: fcron.c,v 1.82 2007/11/07 09:15:40 thib Exp thib $ */
+ /* $Id: fcron.c,v 1.82 2007-11-07 09:15:40 thib Exp $ */
 
 #include "fcron.h"
 
@@ -33,7 +33,7 @@
 #include "socket.h"
 #endif
 
-char rcs_info[] = "$Id: fcron.c,v 1.82 2007/11/07 09:15:40 thib Exp thib $";
+char rcs_info[] = "$Id: fcron.c,v 1.82 2007-11-07 09:15:40 thib Exp $";
 
 void main_loop(void);
 void check_signal(void);
@@ -528,7 +528,8 @@ main(int argc, char **argv)
 	die_e("Could not change dir to %s", fcrontabs);
     
 
-    freopen("/dev/null", "r", stdin);
+    if ( freopen("/dev/null", "r", stdin) == NULL )
+        error_e("Could not open /dev/null as stdin");
 
     if (foreground == 0) {
 
@@ -562,8 +563,10 @@ main(int argc, char **argv)
 	    close(fd);
 	}
 	
-	freopen("/dev/null", "w", stdout);
-	freopen("/dev/null", "w", stderr);
+        if ( freopen("/dev/null", "w", stdout) == NULL )
+            error_e("Could not open /dev/null as stdout");
+        if ( freopen("/dev/null", "w", stderr) == NULL )
+            error_e("Could not open /dev/null as stderr");
 
 	/* close most other open fds */
 	xcloselog();
