@@ -1,7 +1,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2007 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2010 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: fcrondyn.c,v 1.19 2007/04/14 18:04:11 thib Exp thib $ */
+ /* $Id: fcrondyn.c,v 1.19 2007-04-14 18:04:11 thib Exp $ */
 
 /* fcrondyn : interact dynamically with running fcron process :
  *     - list jobs, with their status, next time of execution, etc
@@ -34,7 +34,7 @@
 #include "allow.h"
 #include "read_string.h"
 
-char rcs_info[] = "$Id: fcrondyn.c,v 1.19 2007/04/14 18:04:11 thib Exp thib $";
+char rcs_info[] = "$Id: fcrondyn.c,v 1.19 2007-04-14 18:04:11 thib Exp $";
 
 void info(void);
 void usage(void);
@@ -73,8 +73,9 @@ uid_t rootuid = 0;
 gid_t rootgid = 0;
 
 /* misc */
-char *user_str;
-uid_t user_uid;
+char *user_str = NULL;
+uid_t user_uid = 0;
+gid_t user_gid = 0;
 
 /* if you change this structure, please update NUM_CMD value in dyncom.h */
 struct cmd_list_ent cmd_list[NUM_CMD] = {
@@ -642,31 +643,38 @@ main(int argc, char **argv)
     if ( strrchr(argv[0], '/') == NULL) prog_name = argv[0];
     else prog_name = strrchr(argv[0], '/') + 1;
 
-    /* interpret command line options */
-    parseopt(argc, argv);
-
-/*      if (debug_opt) */
-/*  	fprintf(stderr, "uid : %d euid : %d\n", getuid(), geteuid()); */
-
-    /* read fcron.conf and update global parameters */
-    read_conf();
-
     user_uid = getuid();
+    user_gid = getgid();
     if ( (pass = getpwuid(user_uid)) == NULL )
 	die("user \"%s\" is not in passwd file. Aborting.", USERNAME);
     user_str = strdup2(pass->pw_name);
 
-    /* we don't need anymore special rights : drop them */
-    seteuid(user_uid);
-    setuid(user_uid);
-/*      if (debug_opt) */
-/*  	fprintf(stderr, "uid : %d euid : %d\n", getuid(), geteuid()); */
+    /* drop suid rights that we don't need, but keep the sgid rights
+     * for now as we will need them for read_conf() and is_allowed() */
+#ifdef USE_SETE_ID
+    seteuid_safe(user_uid);
+#endif
+    if ( setuid(user_uid) < 0 )
+        die_e("could not setuid() to %d", user_uid);
+
+    /* interpret command line options */
+    parseopt(argc, argv);
+
+    /* read fcron.conf and update global parameters */
+    read_conf();
 
     if ( ! is_allowed(user_str) ) {
 	die("User \"%s\" is not allowed to use %s. Aborting.",
 	    user_str, prog_name);	    
     }
 
+    /* we don't need anymore special rights : drop remaining ones */
+#ifdef USE_SETE_ID
+    setegid_safe(user_gid);
+#endif
+    if ( setgid(user_gid) < 0 )
+        die_e("could not setgid() to %d", user_gid);
+
     /* check for broken pipes ... */
     signal(SIGPIPE, sigpipe_handler);
 
