@@ -1,7 +1,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2007 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2010 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: fileconf.c,v 1.81 2007/11/07 09:16:40 thib Exp thib $ */
+ /* $Id: fileconf.c,v 1.82 2008-09-21 17:40:34 thib Exp $ */
 
 #include "fcrontab.h"
 
@@ -148,7 +148,7 @@ get_line(char *str, size_t size, FILE *file)
 }
 
 int
-read_file(char *filename)
+read_file(char *filename, int fd)
     /* read file "name" and append cf_t list */
 {
     cf_t *cf = NULL;
@@ -168,15 +168,15 @@ read_file(char *filename)
 
     /* open file */
 
-    /* check if user is allowed to read file */
-    if ( access(file_name, R_OK) != 0 )
-	die_e("User %s can't read file \"%s\"", user, file_name);
-    else if ( (file = fopen(file_name, "r")) == NULL ) {
+    if ( (file = fdopen(fd, "r")) == NULL ) {
 	fprintf(stderr, "Could not open \"%s\": %s\n", file_name,
 		strerror(errno));
 	return ERR;
     }
 
+    /* Rewind, just in case */
+    rewind(file);
+
     Alloc(cf, cf_t);
     cf->cf_user = strdup2(user);
     default_line.cl_file = cf;
@@ -194,7 +194,7 @@ read_file(char *filename)
     /* max_lines acts here as a security counter to avoid endless loop. */
     max_lines = (max_entries * 10) + 10;
 
-    while ( entries <= max_entries && line <= max_lines ) {
+    while ( entries < max_entries && line <= max_lines ) {
 
 	ret = get_line(buf, sizeof(buf), file);
 
@@ -264,7 +264,9 @@ read_file(char *filename)
     cf->cf_next = file_base;
     file_base = cf;
 
-    fclose(file);
+    /* don't close as underlying fd may still be used by calling function */
+    if (fflush(file) != 0)
+        error_e("could not fflush() file_name");
     
     free(default_line.cl_runas);
     free(default_line.cl_mailto);
@@ -297,7 +299,7 @@ read_env(char *ptr, cf_t *cf)
     }
     name[j] = '\0';
 
-    if ( name == '\0' )
+    if ( name[0] == '\0' )
 	goto error;
 
     /* skip '=' and spaces around */
@@ -1717,8 +1719,13 @@ save_file(char *path)
      * ownership to this user, in order to make fcron check the runas fields.
      * (a malicious user could put a runas(root) and wait for the fcrontab to be
      * installed by root) */
+#ifdef USE_SETE_ID
 	if ( save_file_safe(file, path, "fcrontab", asuid, fcrontab_gid, 0) == ERR )
 	    return ERR;
+#else
+	if ( save_file_safe(file, path, "fcrontab", fcrontab_uid, fcrontab_gid, 0) == ERR )
+	    return ERR;
+#endif
 
     }
 
