@@ -1,7 +1,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2007 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2010 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: read_string.c,v 1.4 2007/04/14 18:04:12 thib Exp thib $ */
+ /* $Id: read_string.c,v 1.4 2007-04-14 18:04:12 thib Exp $ */
 
 
 /* read a string (password, etc ...) securely from a tty */
@@ -86,7 +86,7 @@ char *read_string(int echo, const char *prompt)
 	    } else {
 		line[nc] = '\0';
 	    }
-	    input = ( (line) ? strdup(line):NULL );
+	    input = strdup(line);
 	    Overwrite(line);
 
 	    return input;                  /* return malloc()ed string */
