@@ -1,7 +1,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2007 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2010 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: subs.h,v 1.11 2007/04/14 18:04:23 thib Exp thib $ */
+ /* $Id: subs.h,v 1.11 2007-04-14 18:04:23 thib Exp $ */
 
 #ifndef __SUBS_H__
 #define __SUBS_H__
@@ -44,6 +44,12 @@ extern char *sendmail;
 /* functions prototypes */
 extern uid_t get_user_uid_safe(char *username);
 extern gid_t get_group_gid_safe(char *groupname);
+extern void seteuid_safe(uid_t euid);
+extern void setegid_safe(uid_t egid);
+extern int remove_as_user(const char *pathname, uid_t removeuid, gid_t removegid);
+extern int open_as_user(const char *pathname, uid_t openuid, gid_t opengid, int flags, ...);
+extern int rename_as_user(const char *oldpath, const char *newpath, uid_t renameuid, gid_t renamegid);
+
 extern int remove_blanks(char *str);
 extern char *strdup2(const char *);
 extern int get_word(char **str);
